/*
 * Copyright 2013 Mark Slater
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.
 */

package net.sourceforge.xazzle.xhtml;

import java.util.Iterator;
import java.util.NoSuchElementException;

final class IterableUtils {
    static <T> Iterable<T> cons(final T head, final Iterable<? extends T> tail) {
        return new Iterable<T>() {
            @Override
            public Iterator<T> iterator() {
                return new Iterator<T>() {

                    private final Iterator<? extends T> delegate = tail.iterator();
                    private boolean firstUnused = true;

                    @Override
                    public boolean hasNext() {
                        return firstUnused || delegate.hasNext();
                    }

                    @Override
                    public T next() {
                        if (firstUnused) {
                            firstUnused = false;
                            return head;
                        } else {
                            return delegate.next();
                        }
                    }

                    @Override
                    public void remove() {
                        throw new UnsupportedOperationException();
                    }
                };
            }
        };
    }

    static <T> Iterable<T> append(final Iterable<? extends T> list, final T newElement) {
        return new Iterable<T>() {
            @Override
            public Iterator<T> iterator() {
                return new Iterator<T>() {

                    private final Iterator<? extends T> delegate = list.iterator();
                    private boolean lastUnused = true;

                    @Override
                    public boolean hasNext() {
                        return delegate.hasNext() || lastUnused;
                    }

                    @Override
                    public T next() {
                        if (delegate.hasNext()) {
                            return delegate.next();
                        } else {
                            if (!lastUnused) {
                                throw new NoSuchElementException();
                            } else {
                                lastUnused = false;
                                return newElement;
                            }
                        }
                    }

                    @Override
                    public void remove() {
                        throw new UnsupportedOperationException();
                    }
                };
            }
        };
    }

    static <T> Iterable<T> concat(final Iterable<? extends T> first, final Iterable<? extends T> second) {
        return new Iterable<T>() {
            @Override
            public Iterator<T> iterator() {
                return new Iterator<T>() {
                    private final Iterator<? extends T> firstIterator = first.iterator();
                    private final Iterator<? extends T> secondIterator = second.iterator();

                    @Override
                    public boolean hasNext() {
                        return firstIterator.hasNext() || secondIterator.hasNext();
                    }

                    @Override
                    public T next() {
                        if (firstIterator.hasNext()) {
                            return firstIterator.next();
                        } else {
                            return secondIterator.next();
                        }
                    }

                    @Override
                    public void remove() {
                        throw new UnsupportedOperationException();
                    }
                };
            }
        };
    }
}
